%PFE class definition
%
%   Usage:
%      cluster=pfe();
%      cluster=pfe('np',3);
%      cluster=pfe('np',3,'login','username');

classdef pfe
    properties (SetAccess=public)  
		 % {{{1
		 name='pfe'
		 login='elarour';
		 numnodes=20;
		 cpuspernode=8; 
		 port=1025;
		 queue='long';
		 time=12*60;
		 processor='neh';
		 codepath='/staff/elarour/trunk/bin';
		 executionpath='/nobackupp10/elarour/Testing';
		 interactive=0;
	 end
	 properties (SetAccess=private) 
		 np=20*8;
		 % }}}
	 end
	 methods
		 function cluster=pfe(varargin) % {{{1
			 options=pairoptions(varargin{:});
			 for i=1:size(options.list,1),
				 fieldname=options.list{i,1};
				 fieldvalue=options.list{i,2};
				 if ismember(fieldname,properties(pfe)),
					 cluster.(fieldname)=fieldvalue;
				 else
					 disp(['''' fieldname ''' is not a property of cluster pfe']);
				 end
			 end
		 end
		 %}}}
		 function disp(cluster) % {{{1
			 %  display the object
			 disp(sprintf('class ''%s'' object ''%s'' = ',class(cluster),inputname(1)));
			 disp(sprintf('    name: %s',cluster.name));
			 disp(sprintf('    login: %s',cluster.login));
			 disp(sprintf('    port: %i',cluster.port));
			 disp(sprintf('    numnodes: %i',cluster.numnodes));
			 disp(sprintf('    cpuspernode: %i',cluster.cpuspernode));
			 disp(sprintf('    np: %i',cluster.cpuspernode*cluster.numnodes));
			 disp(sprintf('    queue: %s',cluster.queue));
			 disp(sprintf('    time: %i',cluster.time));
			 disp(sprintf('    processor: %s',cluster.processor));
			 disp(sprintf('    codepath: %s',cluster.codepath));
			 disp(sprintf('    executionpath: %s',cluster.executionpath));
			 disp(sprintf('    interactive: %i',cluster.interactive));
		 end
		 %}}}
		 function IsConsistent(cluster) % {{{1

			 available_queues={'long'};
			 queue_requirements_time=[7200];
			 queue_requirements_np=[2048];

			 QueueRequirements(available_queues,queue_requirements_time,queue_requirements_np,cluster.queue,cluster.np,cluster.time)

			 %now, check cluster.cpuspernode according to processor type
			 if (strcmpi(cluster.processor,'har') | strcmpi(cluster.processor,'neh')),
				 if ((cluster.cpuspernode>8 ) | (cluster.cpuspernode<1)),
					 error('IsConsistent error message: cpuspernode should be between 1 and 8 for ''neh'' and ''har'' processors');
				 end
			 elseif strcmpi(cluster.processor,'wes'),
				 if ((cluster.cpuspernode>12 ) | (cluster.cpuspernode<1)),
					 error('IsConsistent error message: cpuspernode should be between 1 and 12 for ''wes'' processors');
				 end
			 else
				 error('IsConsistent error message: unknown processor type, should be ''neh'',''wes'' or ''har''');
			 end

		 end
		 %}}}
		 function BuildQueueScript(cluster,modelname,analysis_type,mem_debug) % {{{1

			 %compute number of processors
			 cluster.np=cluster.numnodes*cluster.cpuspernode;

			 %open file for writing: 
			 fid=fopen([modelname '.queue'],'w');

			 fprintf(fid,'#PBS -S /bin/bash\n');
			 fprintf(fid,'#PBS -N %s\n',modelname);
			 fprintf(fid,'#PBS -l select=%i:ncpus=%i:model=%s\n',cluster.numnodes,cluster.cpuspernode,cluster.processor);
			 fprintf(fid,'#PBS -l walltime=%i\n',cluster.time*60); %walltime is in seconds.
			 fprintf(fid,'#PBS -W group_list=s1010\n');
			 fprintf(fid,'#PBS -m e\n');
			 fprintf(fid,'#PBS -o %s.outlog \n',modelname);
			 fprintf(fid,'#PBS -e %s.errlog \n\n',modelname);

			 fprintf(fid,'. /usr/share/modules/init/bash\n\n');

			 fprintf(fid,'module load comp-intel/11.1.046\n');
			 fprintf(fid,'module load mpi/mpt.1.25\n');
			 fprintf(fid,'module load math/intel_mkl_64_10.0.011\n\n');

			 fprintf(fid,'export PATH="$PATH:."\n\n');
			 fprintf(fid,'export MPI_GROUP_MAX=64\n\n');

			 fprintf(fid,'cd $PBS_O_WORKDIR\n\n');

			 fprintf(fid,'mpiexec -verbose -np %i %s/issm.exe %s $PBS_O_WORKDIR %s.bin %s.petsc %s.outbin %s.lock',cluster.np,cluster.codepath,EnumToString(analysis_type),modelname,modelname,modelname,modelname);

			 %close file
			 fclose(fid);


			 %in interactive mode, create a run file, and errlog and outlog file
			 if cluster.interactive,
				 fid=fopen([modelname '.run'],'w');
				 fprintf(fid,'mpiexec -verbose -np %i %s/issm.exe %s $PBS_O_WORKDIR %s.bin %s.petsc %s.outbin %s.lock',cluster.np,cluster.codepath,EnumToString(analysis_type),modelname,modelname,modelname,modelname);
				 fclose(fid);
				 fid=fopen([modelname '.errlog'],'w');
				 fclose(fid);
				 fid=fopen([modelname '.outlog'],'w');
				 fclose(fid);
			 end
		 end %}}}
		 function LaunchQueueJob(cluster,md,options)% {{{1
			 
			 %lauch command, to be executed via ssh
			 if ~cluster.interactive, 
				launchcommand=['cd ' cluster.executionpath ' && rm -rf ./' md.runtimename ' && mkdir ' md.runtimename ...
			                ' && cd ' md.runtimename ' && mv ../' md.runtimename '.tar.gz ./ && tar -zxf ' md.runtimename '.tar.gz  && qsub ' md.name '.queue '];
			else
				launchcommand=['cd ' cluster.executionpath '/Interactive' num2str(cluster.interactive) ' && tar -zxf ' md.runtimename '.tar.gz'];
			end

			if ~strcmpi(options.batch,'yes'),
				
				%compress the files into one zip.
				compressstring=['tar -zcf ' md.runtimename '.tar.gz ' md.name '.bin ' md.name '.queue '  md.name '.petsc '];
				if md.qmu_analysis,
					compressstring=[compressstring md.name '.qmu.in '];
				end
				if cluster.interactive,
					compressstring=[compressstring md.name '.run ' md.name '.errlog ' md.name '.outlog '];
				end
				system(compressstring);
				
				disp('uploading input file and queueing script');
				if ~cluster.interactive,
					issmscpout(cluster.name,cluster.executionpath,cluster.login,cluster.port,{[md.runtimename '.tar.gz']});
				else
					issmscpout(cluster.name,[cluster.executionpath '/Interactive' num2str(cluster.interactive)],cluster.login,cluster.port,{[md.runtimename '.tar.gz']});
				end
				
				disp('launching solution sequence on remote cluster');
				issmssh(cluster.name,cluster.login,cluster.port,launchcommand);
					
				if cluster.interactive,
					error('done uploading files to interactive directory on remote cluster');
				end

			else
				disp('batch mode requested: not launching job interactively');
				disp('launch solution sequence on remote cluster by hand');
			end
		 end
		 %}}}
		 function Download(cluster,md)% {{{1

			%some check
			if isempty(md.runtimename),
				if ~cluster.interactive,
					error('pfe Download error message: supply runtime name for results to be loaded!');
				end
			end

			%Figure out the  directory where all the files are in: 
			if ~cluster.interactive,
				directory=[cluster.executionpath '/' md.runtimename '/'];
			else
				directory=[cluster.executionpath '/Interactive' num2str(cluster.interactive) '/'];
			end

			%What packages are we picking up from remote cluster
			packages={[md.name '.outlog'],[md.name '.errlog']};
			if md.qmu_analysis,
				packages{end+1}=[md.name '.qmu.err'];
				packages{end+1}=[md.name '.qmu.out'];
				if isfield(md.qmu_params,'tabular_graphics_data'),
					if md.qmu_params.tabular_graphics_data==true,
						packages{end+1}='dakota_tabular.dat'; 
					end
				end
			else
				packages{end+1}=[md.name '.outbin'];
			end

			%copy files from cluster to present directory
			issmscpin(cluster.name, cluster.login, cluster.port, directory, packages);
		end %}}}
	end
end
